﻿using System;
using System.Diagnostics;
using MediaPortal.GUI.Library;
using MediaPortal.Configuration;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace OperaLauncher
{
  [PluginIcons("OperaLauncher.opera-logo.png", "OperaLauncher.opera-logo_disabled.png")]

  public class OperaLauncher : GUIWindow, ISetupForm
  {
    public const string PLUGIN_NAME = "Opera Mobile Launcher";
    public string Path;
    public bool noFullscreen;
    public bool TouchInferface;
    public string Arguments;
    public bool UseButtonForwarding;

    [DllImport("user32.dll")]
    public static extern int FindWindow(
    string lpClassName, // class name
    string lpWindowName // window name
    );

    [DllImport("user32.dll")]
    public static extern int SendMessage(
    int hWnd, // handle to destination window
    uint Msg, // message
    int wParam, // first message parameter
    int lParam // second message parameter
    );

    const int WM_KEYDOWN = 0x0100;
    const int WM_KEYUP = 0x0101;
    //const int WM_APPCOMMAND = 0x0319;
    const int VK_F1 = 0x70;
    const int VK_F2 = 0x71;
    const int VK_NUMPAD0 = 0x60;
    const int VK_NUMPAD1 = 0x61;
    const int VK_NUMPAD2 = 0x62;
    const int VK_NUMPAD3 = 0x63;
    const int VK_NUMPAD4 = 0x64;
    const int VK_NUMPAD5 = 0x65;
    const int VK_NUMPAD6 = 0x66;
    const int VK_NUMPAD7 = 0x67;
    const int VK_NUMPAD8 = 0x68;
    const int VK_NUMPAD9 = 0x69;
    //Begin Call with parameter
    public static bool ParameterSupported = false;
    //public static string Parameter = string.Empty;
    //End Call with parameter

    ProcessStartInfo startInfo = new ProcessStartInfo();
    Process opera = new Process();
    public OperaLauncher()
    {
      using (MediaPortal.Profile.Settings xmlReader = new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, "MediaPortal.xml")))
      {
        Path = xmlReader.GetValueAsString(OperaLauncher.PLUGIN_NAME, "Path", "");
        noFullscreen = xmlReader.GetValueAsBool(OperaLauncher.PLUGIN_NAME, "noFullscreen", false);
        TouchInferface = xmlReader.GetValueAsBool(OperaLauncher.PLUGIN_NAME, "TouchInferface", false);
        Arguments = xmlReader.GetValueAsString(OperaLauncher.PLUGIN_NAME, "Arguments", "");
        UseButtonForwarding = xmlReader.GetValueAsBool(OperaLauncher.PLUGIN_NAME, "UseButtonForwarding", true);
      }
    }

    protected override void OnPageLoad()
    {
      //Begin Call with parameter
      string strUrl = string.Empty;
      if (ParameterSupported)
        strUrl = _loadParameter.Trim();
      if ((null == strUrl) || (string.IsNullOrEmpty(strUrl)))
        strUrl = GUIPropertyManager.GetProperty("#browserlink");
      //string strUrl = GUIPropertyManager.GetProperty("#browserlink");
      //End Call with parameter
      startOpera(strUrl);

      if(!UseButtonForwarding)
        GUIWindowManager.ShowPreviousWindow();
      base.OnPageLoad();
    }

    protected override void OnPageDestroy(int newWindowId)
    {
      //close opera
      opera.CloseMainWindow();
    }

    public void startOpera(string strUrl = null)
    {
      string arguments = "";
      if (!noFullscreen)
        arguments += " -fullscreen";
      if (TouchInferface)
        arguments += " -tabletui";
      else
        arguments += " -notouchwithtouchevents";

      arguments += " " + Arguments;

      opera.StartInfo.FileName = @Path;
      opera.StartInfo.WorkingDirectory = Directory.GetParent(Path).FullName;
      if (strUrl != null)
        opera.StartInfo.Arguments = arguments + " -url " + strUrl;
      else
        opera.StartInfo.Arguments = arguments;

      try
      {
       opera.Start();
      }
      catch (Exception ex)
      {
        Log.Error("Start Opera failed: " + ex.Message);
      }
    }


    public override void OnAction(MediaPortal.GUI.Library.Action action)
    {
      if (UseButtonForwarding)
      {
        int WindowToFind = FindWindow("QWidget", null);

        switch (action.wID)
        {
          case MediaPortal.GUI.Library.Action.ActionType.ACTION_CONTEXT_MENU:
            GUIPropertyManager.SetProperty("#keypressed", "Info");
            SendMessage(WindowToFind, WM_KEYDOWN, VK_F1, 0);
            SendMessage(WindowToFind, WM_KEYUP, VK_F1, 0);
            break;

          case MediaPortal.GUI.Library.Action.ActionType.ACTION_PREV_ITEM:
            GUIPropertyManager.SetProperty("#keypressed", "Replay");
            SendMessage(WindowToFind, WM_KEYDOWN, VK_F2, 0);
            SendMessage(WindowToFind, WM_KEYUP, VK_F2, 0);
            break;

          case MediaPortal.GUI.Library.Action.ActionType.ACTION_PAGE_UP:
            GUIPropertyManager.SetProperty("#keypressed", "Channel Up");
            SendMessage(WindowToFind, WM_KEYDOWN, VK_NUMPAD8, 0);
            SendMessage(WindowToFind, WM_KEYUP, VK_NUMPAD8, 0);
            break;

          case MediaPortal.GUI.Library.Action.ActionType.ACTION_PAGE_DOWN:
            GUIPropertyManager.SetProperty("#keypressed", "Channel Down");
            SendMessage(WindowToFind, WM_KEYDOWN, VK_NUMPAD2, 0);
            SendMessage(WindowToFind, WM_KEYUP, VK_NUMPAD2, 0);
            break;

          case MediaPortal.GUI.Library.Action.ActionType.REMOTE_0:
            GUIPropertyManager.SetProperty("#keypressed", "0");
            SendMessage(WindowToFind, WM_KEYDOWN, VK_NUMPAD0, 0);
            SendMessage(WindowToFind, WM_KEYUP, VK_NUMPAD0, 0);
            break;
          case MediaPortal.GUI.Library.Action.ActionType.REMOTE_1:
            GUIPropertyManager.SetProperty("#keypressed", "1");
            SendMessage(WindowToFind, WM_KEYDOWN, VK_NUMPAD1, 0);
            SendMessage(WindowToFind, WM_KEYUP, VK_NUMPAD1, 0);
            break;
        }
      }
      base.OnAction(action);
    }


    public override bool Init() //bij het laden van MediaPortal
    {
      Log.Debug("Init Opera Mobile Launcher");
      // Begin Call with parameter
      ParameterSupported = false;
      if (typeof(GUIWindow).GetField("_loadParameter", System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance) != null)
      {
          ParameterSupported = true;
      } 
      // End Call with parameter
      if (UseButtonForwarding)
        return Load(GUIGraphicsContext.Skin + @"\OperaMobile.xml");
      else
        return true;
    }

    #region ISetupForm Members

    // Returns the name of the plugin which is shown in the plugin menu
    public string PluginName()
    {
      return PLUGIN_NAME;
    }

    // Returns the description of the plugin is shown in the plugin menu
    public string Description()
    {
      return "Starts Opera Mobile Emulator from MediaPortal";
    }

    // Returns the author of the plugin which is shown in the plugin menu
    public string Author()
    {
      return "Dragy";
    }

    // show the setup dialog
    public void ShowPlugin()
    {
      FormSettings frm = new FormSettings();
      frm.ShowDialog();
    }

    // Indicates whether plugin can be enabled/disabled
    public bool CanEnable()
    {
      return true;
    }

    // Get Windows-ID
    public int GetWindowId()
    {
      // WindowID of windowplugin belonging to this setup
      // enter your own unique code
      return 156778;
    }

    // Indicates if plugin is enabled by default;
    public bool DefaultEnabled()
    {
      return true;
    }

    // indicates if a plugin has it's own setup screen
    public bool HasSetup()
    {
      return true;
    }

    /// <summary>
    /// If the plugin should have it's own button on the main menu of MediaPortal then it
    /// should return true to this method, otherwise if it should not be on home
    /// it should return false
    /// </summary>
    /// <param name="strButtonText">text the button should have</param>
    /// <param name="strButtonImage">image for the button, or empty for default</param>
    /// <param name="strButtonImageFocus">image for the button, or empty for default</param>
    /// <param name="strPictureImage">subpicture for the button or empty for none</param>
    /// <returns>true : plugin needs it's own button on home
    /// false : plugin does not need it's own button on home</returns>

    public bool GetHome(out string strButtonText, out string strButtonImage, out string strButtonImageFocus, out string strPictureImage)
    {
      strButtonText = "Opera Mobile";
      strButtonImage = String.Empty;
      strButtonImageFocus = String.Empty;
      strPictureImage = String.Empty;
      return true;
    }

    // With GetID it will be an window-plugin / otherwise a process-plugin
    // Enter the id number here again
    public override int GetID
    {
      get
      {
        return 156778;
      }

      set
      {
      }
    }

    #endregion

  }
}